#include <Wire.h>
#include <SPI.h>
#include <ModbusMaster.h>

#define USE_UI    //if you want to use the ui export from Squareline, please do not annotate this line.

#define SDA 19
#define SCL 20

#if defined USE_UI
#include <lvgl.h>
#include "ui.h"
#endif

#include <Arduino_GFX_Library.h>
#define TFT_BL -1
#define GFX_BL DF_GFX_BL // default backlight pin, you may replace DF_GFX_BL to actual backlight pin

/******Please define a corresponding line based on your development board.************/
#define Display_50
#define PCA9538_ADDR 0x73
#define PCA9538_INPUT_REG 0x00
#define PCA9538_OUTPUT_REG 0x01
#define PCA9538_POLARITY_REG 0x02
#define PCA9538_CONFIG_REG 0x03

#define FC    -1
#define RX_PIN 48 
#define TX_PIN 2

#define GPIO5 0x10
#define GPIO6 0x20
#define GPIO7 0x40
#define GPIO8 0x80
/*******************************************************************************
 * Screen Driver Configuration 
*******************************************************************************/
void setPinMode(uint8_t pin, uint8_t mode) {
  uint8_t config = readRegister(PCA9538_CONFIG_REG);
  if (mode == INPUT) {
    config |= pin;
  } else {
    config &= ~pin;
  }
  writeRegister(PCA9538_CONFIG_REG, config);
}

void writeOutput(uint8_t pin, uint8_t value) {
  uint8_t output = readRegister(PCA9538_OUTPUT_REG);
  if (value == LOW) {
    output &= ~pin;
  } else {
    output |= pin;
  }
  writeRegister(PCA9538_OUTPUT_REG, output);
}
uint8_t readInput() {
  return readRegister(PCA9538_INPUT_REG);
}

uint8_t readRegister(uint8_t reg) {
  Wire.beginTransmission(PCA9538_ADDR);
  Wire.write(reg);
  Wire.endTransmission(false);
  Wire.requestFrom(PCA9538_ADDR, 1);
  return Wire.read();
}
void writeRegister(uint8_t reg, uint8_t value) {
  Wire.beginTransmission(PCA9538_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission();
}

ModbusMaster node;
int LED1;
int LED2;
int LED3;
int LED4;

void preTransmission()
{
  digitalWrite(FC, 1);
}

void postTransmission()
{
  digitalWrite(FC, 0);
}

Arduino_ESP32RGBPanel *bus = new Arduino_ESP32RGBPanel(
    GFX_NOT_DEFINED /* CS */, GFX_NOT_DEFINED /* SCK */, GFX_NOT_DEFINED /* SDA */,
    4 /* DE */, 5 /* VSYNC */, 6 /* HSYNC */, 7 /* PCLK */,
    1 /* R0 */, 41 /* R1 */, 40 /* R2 */, 38 /* R3 */, 45 /* R4 */,
    47 /* G0 */, 21 /* G1 */, 14 /* G2 */, 9 /* G3 */, 3 /* G4 */, 3 /* G5 */,
    8 /* B0 */, 18 /* B1 */, 17 /* B2 */, 16 /* B3 */, 15 /* B4 */
);
Arduino_RPi_DPI_RGBPanel *lcd = new Arduino_RPi_DPI_RGBPanel(
    bus,
    800 /* width */, 0 /* hsync_polarity */, 210 /* hsync_front_porch */, 30 /* hsync_pulse_width */, 16 /* hsync_back_porch */,
    480 /* height */, 0 /* vsync_polarity */, 22 /* vsync_front_porch */, 13 /* vsync_pulse_width */, 10 /* vsync_back_porch */,
    1 /* pclk_active_neg */, 16000000 /* prefer_speed */, true /* auto_flush */);
  


/*******************************************************************************
 * Screen Driver Configuration  end
*******************************************************************************/


/*******************************************************************************
   Please config the touch panel in touch.h
 ******************************************************************************/
#include "touch.h"

#ifdef USE_UI
/* Change to your screen resolution */
static uint32_t screenWidth;
static uint32_t screenHeight;
static lv_disp_draw_buf_t draw_buf;
static lv_color_t disp_draw_buf[800 * 480 / 10];      //5,7inch: lv_color_t disp_draw_buf[800*480/10]            4.3inch: lv_color_t disp_draw_buf[480*272/10]
//static lv_color_t disp_draw_buf;
static lv_disp_drv_t disp_drv;

/* Display flushing */
void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p)
{
  uint32_t w = (area->x2 - area->x1 + 1);
  uint32_t h = (area->y2 - area->y1 + 1);

#if (LV_COLOR_16_SWAP != 0)
  lcd->draw16bitBeRGBBitmap(area->x1, area->y1, (uint16_t *)&color_p->full, w, h);
#else
  lcd->draw16bitRGBBitmap(area->x1, area->y1, (uint16_t *)&color_p->full, w, h);
#endif

  lv_disp_flush_ready(disp);
}

void my_touchpad_read(lv_indev_drv_t *indev_driver, lv_indev_data_t *data)
{
  if (touch_has_signal())
  {
    if (touch_touched())
    {
      data->state = LV_INDEV_STATE_PR;

      /*Set the coordinates*/
      data->point.x = touch_last_x;
      data->point.y = touch_last_y;
      Serial.print( "Data x :" );
      Serial.println( touch_last_x );

      Serial.print( "Data y :" );
      Serial.println( touch_last_y );
    }
    else if (touch_released())
    {
      data->state = LV_INDEV_STATE_REL;
    }
  }
  else
  {
    data->state = LV_INDEV_STATE_REL;
  }
  // delay(15);
}
#endif

uint8_t result;
void button1_on(lv_event_t * e)
{
  LED1 = 1;
	writeOutput(GPIO5, 1);
  result = node.writeSingleCoil(0x00001, LED1);
}

void button1_off(lv_event_t * e)
{
  LED1 = 0;
	writeOutput(GPIO5, 0);
  result = node.writeSingleCoil(0x00001, LED1);
}

void button2_on(lv_event_t * e)
{
  LED2 = 1;
  writeOutput(GPIO6, 1);
  result = node.writeSingleCoil(0x00002, LED2);
}

void button2_off(lv_event_t * e)
{
  LED2 = 0;
  writeOutput(GPIO6, 0);
  result = node.writeSingleCoil(0x00002, LED2);
}

void button3_on(lv_event_t * e)
{
  LED3 = 1;
  writeOutput(GPIO7, 1);
  result = node.writeSingleCoil(0x00003, LED3);
}

void button3_off(lv_event_t * e)
{
  LED3 = 0;
  writeOutput(GPIO7, 0);
  result = node.writeSingleCoil(0x00003, LED3);
}

void button4_on(lv_event_t * e)
{
  LED4 = 1;
  writeOutput(GPIO8, 1);
  result = node.writeSingleCoil(0x00004, LED4);
}

void button4_off(lv_event_t * e)
{
  LED4 = 0;
  writeOutput(GPIO8, 0);
  result = node.writeSingleCoil(0x00004, LED4);
}

void setup()
{
  Serial.begin(9600);
  Serial.println("LVGL Widgets Demo");

  // pinMode(FC, OUTPUT);
  // digitalWrite(FC, 0);
  
  Serial1.begin(9600, SERIAL_8N1, RX_PIN, TX_PIN);
  node.begin(5, Serial1);                           //Slave ID as 1
  node.preTransmission(preTransmission);
  node.postTransmission(postTransmission);

// #if defined(Display_50) || defined(Display_70)
//   //IO Port Pins
//   pinMode(38, OUTPUT);
//   digitalWrite(38, LOW);
//   pinMode(17, OUTPUT);
//   digitalWrite(17, LOW);
//   pinMode(18, OUTPUT);
//   digitalWrite(18, LOW);
//   pinMode(42, OUTPUT);
//   digitalWrite(42, LOW);
// #elif defined(Display_43)
//   pinMode(20, OUTPUT);
//   digitalWrite(20, LOW);
//   pinMode(19, OUTPUT);
//   digitalWrite(19, LOW);
//   pinMode(35, OUTPUT);
//   digitalWrite(35, LOW);
//   pinMode(38, OUTPUT);
//   digitalWrite(38, LOW);
//   pinMode(0, OUTPUT);//TOUCH-CS
// #endif

  // Init Display
  lcd->begin();
  lcd->fillScreen(BLACK);
  lcd->setTextSize(2);
  // delay(20);
  
#ifdef USE_UI
  lv_init();

  // delay(100);
  touch_init();

  screenWidth = lcd->width();
  screenHeight = lcd->height();

  lv_disp_draw_buf_init(&draw_buf, disp_draw_buf, NULL, screenWidth * screenHeight / 10);
  //  lv_disp_draw_buf_init(&draw_buf, disp_draw_buf, NULL, 480 * 272 / 10);
  /* Initialize the display */
  lv_disp_drv_init(&disp_drv);
  /* Change the following line to your display resolution */
  disp_drv.hor_res = screenWidth;
  disp_drv.ver_res = screenHeight;
  disp_drv.flush_cb = my_disp_flush;
  disp_drv.draw_buf = &draw_buf;
  lv_disp_drv_register(&disp_drv);

  /* Initialize the (dummy) input device driver */
  static lv_indev_drv_t indev_drv;
  lv_indev_drv_init(&indev_drv);
  indev_drv.type = LV_INDEV_TYPE_POINTER;
  indev_drv.read_cb = my_touchpad_read;
  lv_indev_drv_register(&indev_drv);
#endif
  
#ifdef TFT_BL
  pinMode(TFT_BL, OUTPUT);
  digitalWrite(TFT_BL, HIGH);
#endif

#ifdef USE_UI
  ui_init();//ui from Squareline or GUI Guider
#else
  lcd->fillScreen(RED);
  // delay(80);
  lcd->fillScreen(BLUE);
  // delay(80);
  lcd->fillScreen(YELLOW);
  // delay(80);
  lcd->fillScreen(GREEN);
  // delay(80);
#endif
  Serial.println( "Setup done" );

  Wire.begin(SDA, SCL);
  setPinMode(GPIO5, OUTPUT);
  setPinMode(GPIO6, OUTPUT);
  setPinMode(GPIO7, OUTPUT);
  setPinMode(GPIO8, OUTPUT);

}

void loop()
{
  while(1)
  {
#ifdef USE_UI
    lv_timer_handler();
    // delay(5);
#endif
  }
}
